#!/bin/bash -e
###############################################################################
# code's file of arepo plugin for rpmrebuild
#
# There are several required environment variables:
# AREPO_MODE - operation mode, either "lib" or "prog"
# AREPO_PKGLIST - file containing the list of all arepo package names
# AREPO_ARCH - compat architecture name
# AREPO_COMPAT - path to input package of the compat architecture
# AREPO_NATIVE - path to input package of the native architecture, may be unset
# AREPO_CONF - path to config file, defaults to /etc/rpmrebuild-arepo.conf

fatal()
{
	printf >&2 '%s\n' "${0##*/}: $*"
	exit 1
}

FILTER_MODE="${LONG_OPTION#change-spec-}"
[ -n "$FILTER_MODE" -a "$FILTER_MODE" != "$LONG_OPTION" ] ||
	fatal "Invalid LONG_OPTION ($LONG_OPTION)!"

[ -n "$AREPO_CONF" ] ||
	AREPO_CONF=/etc/rpmrebuild-arepo.conf

lib_files=
blacklist_files=

. $AREPO_CONF

[ -n "$AREPO_ARCH" ] ||
	fatal 'AREPO_ARCH is not set!'

# There are two possible arepo-modes: lib (repack only libraries) and prog (repack all)
[ "$AREPO_MODE" = "lib" -o "$AREPO_MODE" = "prog" ] ||
	fatal "Invalid AREPO_MODE ($AREPO_MODE), valid modes are 'lib' and 'prog'."

[ -f "$AREPO_PKGLIST" ] ||
	fatal "Arepo state file $AREPO_PKGLIST is not accessible!"

[ -f "$AREPO_COMPAT" ] ||
	fatal "Donor rpm file $AREPO_COMPAT is not accessible!"

ORIGRPMNAME=`rpmquery --qf "%{NAME}" -p "$AREPO_COMPAT"`

tmpdir=
cleanup_tmpdir()
{
	[ -z "$tmpdir" ] ||
		rm -rf -- "$tmpdir"
	exit "$@"
}

create_tmpdir()
{
	[ -z "$tmpdir" ] ||
		return 0

	tmpdir=$(mktemp -dt "${0##*/}.XXXXXXXX")
	trap 'cleanup_tmpdir $?' EXIT
	trap 'exit 143' HUP INT QUIT PIPE TERM
}

get_tag_depname()
{
	# Fetch NAME from "Tag: NAME..."
	printf '%s\n' "$*" |
		sed -n 's,^[^:]\+: \+\([^ <=>]\+\).*,\1,p'
}

arepoize_tag()
{
	# Convert "Tag: NAME..." -> "Tag: $AREPO_ARCH-NAME..."
	printf '%s\n' "$*" |
		sed 's,^\([^:]\+: \+\)\([^ <=>].*\),\1'"$AREPO_ARCH"'-\2,'
}

is_arepoized_name()
{
	local name="$1"; shift
	[ -n "$name" ] &&
	grep -Fqsx "$AREPO_ARCH-$name" "$AREPO_PKGLIST" &&
		return 0 || return 1
}

output_intact()
{
	printf '%s\n' "$*"
}

filter_description()
{
	echo "[This package was automatically repackaged from $AREPO_ARCH.]"
	echo
	cat
}

# common code for conflicts and obsoletes
filter_obsoletes()
{
	local line DEPNAME
	while read line; do
		arepoize_tag "$line"
		if [ "$AREPO_MODE" = "prog" ]; then
			DEPNAME="$(get_tag_depname "$line")"
			if ! is_arepoized_name "$DEPNAME"; then
				# copy original conflicts/obsoletes
				output_intact "$line"
			fi
		fi
	done
}

filter_conflicts()
{
	if [ "$AREPO_MODE" = "prog" -a -f "$AREPO_NATIVE" ]; then
		# add Conflicts: %name for the native package
		echo "Conflicts: $ORIGRPMNAME"
	fi
	filter_obsoletes
}

filter_files()
{
	if [ "$AREPO_MODE" = "prog" ]; then
		# filter out file names containing Percent symbols
		grep -v '".*%' ||:
		return
	fi

	create_tmpdir

	# dump %files from the native package
	if [ -f "$AREPO_NATIVE" ]; then
		rpmquery -l -p "$AREPO_NATIVE" > "$tmpdir/orig"
	else
		: > "$tmpdir/orig"
	fi
	sort -t $'\t' -u -o "$tmpdir"/orig{,}

	# filter out file names containing Tab and Percent symbols
	# input format: ..."${file}"
	# output format: ${file}\t...
	sed -n 's,^\([^"\t]*\)"\(/[^%\t]*\)"$,\2\t\1,p' > "$tmpdir"/ours
	sort -t $'\t' -u -o "$tmpdir"/ours{,}

	if [ "$AREPO_MODE" = "lib" ]; then
                # filter out %ghost files
		sed -i '/[[:space:]]%ghost[[:space:]]/d' "$tmpdir"/ours
	fi

	cut -f1 "$tmpdir"/ours | # take package names
		grep -Ee "$lib_files" | # filter them through whitelist
		grep -Eve "$blacklist_files" | # filter them through blacklist_files
		comm -23 - "$tmpdir/orig" | # subtract those that also packaged to the native package
		join -t $'\t' -j 1 -o 2.2,2.1 - "$tmpdir"/ours | # output format: ...\t${file}
		sed 's/\t\(.*\)/"\1"/' # convert to original input format
}

filter_preamble()
{
	# arepoize package name
	sed 's/\(^Name: *\)/\1'"$AREPO_ARCH"'-/'
}

filter_provides()
{
	local line DEPNAME
	while read line; do
		case "$line" in
			*.so.*|*.so|*.so' '*|*.so'('*')'*)
				# a soname provide
				output_intact "$line"
				;;
			*)
				DEPNAME="$(get_tag_depname "$line")"
				if [ "$DEPNAME" = "$ORIGRPMNAME" ]; then
					# keeping self-provide as sometimes we
					# provide ourselves with different
					# version (not filtering standard NEVR
					# so we can't get rid of it anyway)
					arepoize_tag "$line"
				fi
				if [ "$AREPO_MODE" = "prog" ]; then
					# copy all the rest in prog mode intact
					output_intact "$line"
				fi
				if is_arepoized_name "$DEPNAME"; then
					# there is an arepoized package
					# corresponding to this provide so there
					# might be an arepoized requirement
					# corresponding to this arepoized provide.
					arepoize_tag "$line"
				fi
				;;
		esac
	done
}

filter_requires()
{
	if [ "$AREPO_MODE" = "lib" ]; then
		# add Requires: %name=EVR for the native package
		ORIGRPMREQ=`rpmquery --qf "%{NAME} = %|EPOCH?{%{EPOCH}:}|%{VERSION}-%{RELEASE}" -p "$AREPO_COMPAT"`
		if [ -f "$AREPO_NATIVE" ]; then
			echo "Requires: $ORIGRPMREQ"
		fi
	fi

	local line DEPNAME
	while read line; do
		case "$line" in
			*.so.*|*.so|*.so' '*)
				# a soname req
				output_intact "$line"
				;;
			*)
				DEPNAME="$(get_tag_depname "$line")"
				if is_arepoized_name "$DEPNAME"; then
					# there is an arepoized package corresponding to this req
					arepoize_tag "$line"
				elif [ "$AREPO_MODE" = "prog" ]; then
					# copy all the rest in prog mode intact
					output_intact "$line"
				fi
				;;
		esac
	done
}

filter_scripts()
{
	if [ "$AREPO_MODE" = 'prog' ]; then
		# in prog mode copy scripts as is
		cat
	elif [ "$AREPO_MODE" = 'lib' -a -f "$AREPO_NATIVE" ]; then
		# copy scripts iff they differ
		local t qf
		case "$FILTER_MODE" in
			pre) t=PREIN ;;
			post) t=POSTIN ;;
			preun) t=PREUN ;;
			postun) t=POSTUN ;;
			triggers ) qf='[trigger%{TRIGGERTYPE} script (through %{TRIGGERSCRIPTPROG}) -- %{TRIGGERCONDS}\n%{TRIGGERSCRIPTS}\n]' ;;
			*) # discard unrecognized sections
			cat > /dev/null
			return ;;
		esac
		[ -n "$qf" ] ||
			qf="'%|${t}?{preinstall scriptlet%|${t}PROG?{ (through %{${t}PROG})}|:\\n%{${t}}\\n}:{%|${t}PROG?{preinstall program: %{${t}PROG}\\n}|}|'"
		create_tmpdir
		rpmquery --qf "$qf" -p "$AREPO_COMPAT" > "$tmpdir/script_compat"
		rpmquery --qf "$qf" -p "$AREPO_NATIVE" > "$tmpdir/script_native"
		if cmp -s "$tmpdir/script_compat" "$tmpdir/script_native"; then
			cat > /dev/null
		else
			cat
		fi
	else
		# discard
		cat > /dev/null
	fi
}

case "$FILTER_MODE" in
	conflicts|description|files|obsoletes|preamble|provides|requires)
		filter_$FILTER_MODE
		;;
	triggers|pre|post|preun|postun)
		filter_scripts
		;;
	*)
		cat
		;;
esac
